==> spellathon-5.47.tar.gz: spellathon-5.47/src/gui/puzzle.cc <==
/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <cmath>

#include <QMouseEvent>
#include <QPaintEvent>
#include <QPainter>
#include <QLineEdit>

#include "puzzle.h"

Puzzle::Puzzle(QWidget *p) 
	: QWidget(p), sd(85.0), sd2(32.0)
{
	double sqrt3 = sqrt(3);
	ty = (sd-sd2)*0.5*sqrt3;
	tx = sd-sd2;

	double le_pos[7][2] = {
  		{ 2.0*sd-114.0-sd2	, 35					},
  		{ sd-sd2/2			, 8						},
		{ 114				, 35					}, 
		{ 114				, sd*sqrt3-35-sd2		}, 
		{ sd-sd2/2			, sd*sqrt3-sd2-8		},
		{ 2*sd-114-sd2		, sd*sqrt3-35-sd2		},
		{ sd-sd2/2			, sd*sqrt3/2.0-sd2/2.0	}
	};

	points[0] = QPointF( 1.0		, sd*0.5*sqrt3	);
	points[1] =	QPointF( sd*0.5		, 1.0			);
	points[2] =	QPointF( sd*0.5+sd	, 1.0			);
	points[3] =	QPointF( sd*2 - 1.0	, sd*0.5*sqrt3	);
	points[4] =	QPointF( sd*0.5+sd	, sd*sqrt3		);
	points[5] =	QPointF( sd*0.5		, sd*sqrt3		);

	points2[0] = QPointF(tx + 0.0			, ty + sd2*0.5*sqrt3);
	points2[1] = QPointF(tx + sd2*0.5		, ty + 0.0			);
	points2[2] = QPointF(tx + sd2*0.5+sd2	, ty + 0.0			);
	points2[3] = QPointF(tx + sd2*2			, ty + sd2*0.5*sqrt3);
	points2[4] = QPointF(tx + sd2*0.5+sd2	, ty + sd2*sqrt3	);
	points2[5] = QPointF(tx + sd2*0.5		, ty + sd2*sqrt3	);

	QVector<QPoint>	b1, b2, b3, b4, b5, b6, b7;

	b1.append(points[0].toPoint());		
	b1.append(points[1].toPoint());
	b1.append(points2[1].toPoint());	
	b1.append(points2[0].toPoint());
	b2.append(points[1].toPoint());		
	b2.append(points[2].toPoint());
	b2.append(points2[2].toPoint());	
	b2.append(points2[1].toPoint());
	b3.append(points[2].toPoint());		
	b3.append(points[3].toPoint());
	b3.append(points2[3].toPoint());	
	b3.append(points2[2].toPoint());
	b4.append(points[3].toPoint());		
	b4.append(points[4].toPoint());
	b4.append(points2[4].toPoint());	
	b4.append(points2[3].toPoint());
	b5.append(points[4].toPoint());		
	b5.append(points[5].toPoint());
	b5.append(points2[5].toPoint());	
	b5.append(points2[4].toPoint());
	b6.append(points[5].toPoint());		
	b6.append(points[0].toPoint());
	b6.append(points2[0].toPoint());	
	b6.append(points2[5].toPoint());
	b7.append(points2[0].toPoint());	
	b7.append(points2[1].toPoint());
	b7.append(points2[2].toPoint());	
	b7.append(points2[3].toPoint());
	b7.append(points2[4].toPoint());	
	b7.append(points2[5].toPoint());

	p1 = new QPolygon(b1);
	p2 = new QPolygon(b2);
	p3 = new QPolygon(b3);
	p4 = new QPolygon(b4);
	p5 = new QPolygon(b5);
	p6 = new QPolygon(b6);
	p7 = new QPolygon(b7);

	QPointF points3[6];
	points3[0] = QPointF( -2.0			, sd*0.5*sqrt3	);
	points3[1] = QPointF( sd*0.5 - 2.0	, -2.0			);
	points3[2] = QPointF( sd*0.5+sd+2.0	, -2.0			);
	points3[3] = QPointF( sd*2 + 2.0	, sd*0.5*sqrt3	);
	points3[4] = QPointF( sd*0.5+sd+2.0	, sd*sqrt3 + 2.0);
	points3[5] = QPointF( sd*0.5 - 2.0	, sd*sqrt3 + 2.0);

	QVector<QPoint> outline;
	outline.append(points3[0].toPoint());
	outline.append(points3[1].toPoint());
	outline.append(points3[2].toPoint());
	outline.append(points3[3].toPoint());
	outline.append(points3[4].toPoint());
	outline.append(points3[5].toPoint());

	setMask(QRegion(QPolygon(outline)));
	setCursor(Qt::IBeamCursor);

	for (int i = 0; i < 7; i++) {
		le[i] = new Ledit(this);
		le[i]->setMaxLength(1);
		le[i]->setAlignment(Qt::AlignHCenter);
		le[i]->resize(32,32);
		le[i]->setStyleSheet("border: none; color: black; background-color: snow;"
			   				 "selection-color: white; selection-background-color: black");
		connect(le[i],  SIGNAL(textEdited(QString)), this, SLOT(toUpper()));
		le[i]->setContextMenuPolicy(Qt::NoContextMenu); 
		le[i]->move((int)le_pos[i][0], (int)le_pos[i][1]);
		le[i]->show();
 	}
	le[6]->setStyleSheet("background: black; color: white;"
						 "border: 2px solid black; selection-color: black;"
						 "selection-background-color: white");

	setTabOrder(le[5], le[6]);
	setTabOrder(le[6], le[0]);
	setTabOrder(le[0], le[1]);
	setTabOrder(le[1], le[2]);
	setTabOrder(le[2], le[3]);
	setTabOrder(le[3], le[4]);
	setTabOrder(le[4], le[5]);

	setMinimumSize(QSize(int(2*sd)+4, int(sd*sqrt3)+4));
	setMaximumSize(QSize(int(2*sd)+4, int(sd*sqrt3)+4));
}

Puzzle::~Puzzle()
{
	delete p1;
	delete p2;
	delete p3;
	delete p4;
	delete p5;
	delete p6;
	delete p7;
}

void
Puzzle::setLetrs(QString s)
{
	s = s.toUpper();

	for (int i = 0; i < 7; i++) 
		le[i]->setText(QString(s[i]));
}
 
void
Puzzle::setReadOnly(bool r)
{
	Qt::FocusPolicy fp = r ? Qt::NoFocus : Qt::StrongFocus;
	QString bc = r ? "QLineEdit { border: none; color: black; background-color: snow;"
			   		 "selection-color: black; selection-background-color: snow }"
					:"QLineEdit { border: none; color: black; background-color: snow;"
			   		 "selection-color: white; selection-background-color: black }";

	r ? setCursor(Qt::ArrowCursor) : setCursor(Qt::IBeamCursor);

	for (int i = 0; i < 7; i++) {
		le[i]->setReadOnly(r); 
		le[i]->deselect(); 
		le[i]->setFocusPolicy(fp); 
		le[i]->setStyleSheet(bc);
		if (r) 
			le[i]->setCursor(Qt::ArrowCursor);
		else
			le[i]->setCursor(Qt::IBeamCursor);
	}
	if (r)
		le[6]->setStyleSheet("background: black; color: white;"
							 "border: 2px solid black; selection-color: white;"
							 "selection-background-color: black");
	else
		le[6]->setStyleSheet("background: black; color: white;"
							 "border: 2px solid black; selection-color: black;"
							 "selection-background-color: white");
		
}

void
Puzzle::toUpper()
{ 
	for (int i = 0; i < 7; i++) {
		le[i]->setText(le[i]->text().toUpper()); 
		le[i]->setModified(true);
	}
}

QString
Puzzle::letrs() const
{ 
	QString puzzle;

	for (int i = 0; i < 7; i++) {
		if (!le[i]->isModified())
			return "";

		puzzle[i] = (le[i]->text())[0].toLower();
		if (!puzzle[i].isLower()) 
			return "";
	}

	return puzzle;
}

void 
Puzzle::paintEvent(QPaintEvent *ev)
{
	QPainter p(this);
	p.setRenderHints(QPainter::Antialiasing);
	p.setBrush(QBrush(QColor("snow")));
	p.drawPolygon(points, 6);		
	p.setBrush(QBrush(Qt::black));
	p.drawPolygon(points2, 6);		

	for (int i = 0; i < 6; i++)
		p.drawLine(QLineF(points[i], points2[i]));

	ev->accept();
}

void 
Puzzle::mousePressEvent(QMouseEvent *ev)
{
	const QPoint pt = ev->pos();
	if (p1->containsPoint(pt, Qt::OddEvenFill)) {
		le[0]->setFocus();
		le[0]->selectAll();
	} else if (p2->containsPoint(pt, Qt::OddEvenFill)) {
		le[1]->setFocus();
		le[1]->selectAll();
	} else if (p3->containsPoint(pt, Qt::OddEvenFill)) {
		le[2]->setFocus();
		le[2]->selectAll();
	} else if (p4->containsPoint(pt, Qt::OddEvenFill)) {
		le[3]->setFocus();
		le[3]->selectAll();
	} else if (p5->containsPoint(pt, Qt::OddEvenFill)) {
		le[4]->setFocus();
		le[4]->selectAll();
	} else if (p6->containsPoint(pt, Qt::OddEvenFill)) {
		le[5]->setFocus();
		le[5]->selectAll();
	} else if (p7->containsPoint(pt, Qt::OddEvenFill)) {
		le[6]->setFocus();
		le[6]->selectAll();
	}

	ev->accept();
}

==> spellathon-5.59.tar.gz: spellathon-5.59/src/gui/puzzle.cc <==
/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <cmath>

#include <QMouseEvent>
#include <QPaintEvent>
#include <QPainter>
#include <QLineEdit>

#include "puzzle.h"

Puzzle::Puzzle(QWidget *p) 
	: QWidget(p), sd(85.0), sd2(32.0)
{
	double sqrt3 = sqrt(3);
	ty = (sd-sd2)*0.5*sqrt3;
	tx = sd-sd2;

	double le_pos[7][2] = {
  		{ 2.0*sd-114.0-sd2	, 35					},
  		{ sd-sd2/2			, 8						},
		{ 114				, 35					}, 
		{ 114				, sd*sqrt3-35-sd2		}, 
		{ sd-sd2/2			, sd*sqrt3-sd2-8		},
		{ 2*sd-114-sd2		, sd*sqrt3-35-sd2		},
		{ sd-sd2/2			, sd*sqrt3/2.0-sd2/2.0	}
	};

	points[0] = QPointF( 1.0		, sd*0.5*sqrt3	);
	points[1] =	QPointF( sd*0.5		, 1.0			);
	points[2] =	QPointF( sd*0.5+sd	, 1.0			);
	points[3] =	QPointF( sd*2 - 1.0	, sd*0.5*sqrt3	);
	points[4] =	QPointF( sd*0.5+sd	, sd*sqrt3		);
	points[5] =	QPointF( sd*0.5		, sd*sqrt3		);

	points2[0] = QPointF(tx + 0.0			, ty + sd2*0.5*sqrt3);
	points2[1] = QPointF(tx + sd2*0.5		, ty + 0.0			);
	points2[2] = QPointF(tx + sd2*0.5+sd2	, ty + 0.0			);
	points2[3] = QPointF(tx + sd2*2			, ty + sd2*0.5*sqrt3);
	points2[4] = QPointF(tx + sd2*0.5+sd2	, ty + sd2*sqrt3	);
	points2[5] = QPointF(tx + sd2*0.5		, ty + sd2*sqrt3	);

	QVector<QPoint>	b1, b2, b3, b4, b5, b6, b7;

	b1.append(points[0].toPoint());		
	b1.append(points[1].toPoint());
	b1.append(points2[1].toPoint());	
	b1.append(points2[0].toPoint());
	b2.append(points[1].toPoint());		
	b2.append(points[2].toPoint());
	b2.append(points2[2].toPoint());	
	b2.append(points2[1].toPoint());
	b3.append(points[2].toPoint());		
	b3.append(points[3].toPoint());
	b3.append(points2[3].toPoint());	
	b3.append(points2[2].toPoint());
	b4.append(points[3].toPoint());		
	b4.append(points[4].toPoint());
	b4.append(points2[4].toPoint());	
	b4.append(points2[3].toPoint());
	b5.append(points[4].toPoint());		
	b5.append(points[5].toPoint());
	b5.append(points2[5].toPoint());	
	b5.append(points2[4].toPoint());
	b6.append(points[5].toPoint());		
	b6.append(points[0].toPoint());
	b6.append(points2[0].toPoint());	
	b6.append(points2[5].toPoint());
	b7.append(points2[0].toPoint());	
	b7.append(points2[1].toPoint());
	b7.append(points2[2].toPoint());	
	b7.append(points2[3].toPoint());
	b7.append(points2[4].toPoint());	
	b7.append(points2[5].toPoint());

	p1 = new QPolygon(b1);
	p2 = new QPolygon(b2);
	p3 = new QPolygon(b3);
	p4 = new QPolygon(b4);
	p5 = new QPolygon(b5);
	p6 = new QPolygon(b6);
	p7 = new QPolygon(b7);

	QPointF points3[6];
	points3[0] = QPointF( -2.0			, sd*0.5*sqrt3	);
	points3[1] = QPointF( sd*0.5 - 2.0	, -2.0			);
	points3[2] = QPointF( sd*0.5+sd+2.0	, -2.0			);
	points3[3] = QPointF( sd*2 + 2.0	, sd*0.5*sqrt3	);
	points3[4] = QPointF( sd*0.5+sd+2.0	, sd*sqrt3 + 2.0);
	points3[5] = QPointF( sd*0.5 - 2.0	, sd*sqrt3 + 2.0);

	QVector<QPoint> outline;
	outline.append(points3[0].toPoint());
	outline.append(points3[1].toPoint());
	outline.append(points3[2].toPoint());
	outline.append(points3[3].toPoint());
	outline.append(points3[4].toPoint());
	outline.append(points3[5].toPoint());

	setMask(QRegion(QPolygon(outline)));
	setCursor(Qt::IBeamCursor);

	for (int i = 0; i < 7; i++) {
		le[i] = new Ledit(this);
		le[i]->setMaxLength(1);
		le[i]->setAlignment(Qt::AlignHCenter);
		le[i]->resize(32,32);
		le[i]->setStyleSheet("border: none; color: black; background-color: snow;"
			   				 "selection-background-color: #bad8ff; selection-color: black;");
		connect(le[i],  SIGNAL(textEdited(QString)), this, SLOT(toUpper()));
		le[i]->setContextMenuPolicy(Qt::NoContextMenu); 
		le[i]->move((int)le_pos[i][0], (int)le_pos[i][1]);
		le[i]->show();
 	}
	le[6]->setStyleSheet("background: black; color: white;"
						 "border: 2px solid black; selection-background-color: #bad8ff; selection-color: black;");

	setTabOrder(le[5], le[6]);
	setTabOrder(le[6], le[0]);
	setTabOrder(le[0], le[1]);
	setTabOrder(le[1], le[2]);
	setTabOrder(le[2], le[3]);
	setTabOrder(le[3], le[4]);
	setTabOrder(le[4], le[5]);

	setMinimumSize(QSize(int(2*sd)+4, int(sd*sqrt3)+4));
	setMaximumSize(QSize(int(2*sd)+4, int(sd*sqrt3)+4));
}

Puzzle::~Puzzle()
{
	delete p1;
	delete p2;
	delete p3;
	delete p4;
	delete p5;
	delete p6;
	delete p7;
}

void
Puzzle::setLetrs(QString s)
{
	s = s.toUpper();

	for (int i = 0; i < 7; i++) 
		le[i]->setText(QString(s[i]));
}
 
void
Puzzle::setReadOnly(bool r)
{
	Qt::FocusPolicy fp = r ? Qt::NoFocus : Qt::StrongFocus;
	QString bc = r ? "QLineEdit { border: none; color: black; background-color: snow;"
			   		 "selection-color: black; selection-background-color: snow }"
					:"QLineEdit { border: none; color: black; background-color: snow;"
			   		 "selection-background-color: #bad8ff; selection-color: black;}";

	r ? setCursor(Qt::ArrowCursor) : setCursor(Qt::IBeamCursor);

	for (int i = 0; i < 7; i++) {
		le[i]->setReadOnly(r); 
		le[i]->deselect(); 
		le[i]->setFocusPolicy(fp); 
		le[i]->setStyleSheet(bc);
		if (r) 
			le[i]->setCursor(Qt::ArrowCursor);
		else
			le[i]->setCursor(Qt::IBeamCursor);
	}
	if (r)
		le[6]->setStyleSheet("background: black; color: white;"
							 "border: 2px solid black; selection-color: white;"
							 "selection-background-color: black");
	else
		le[6]->setStyleSheet("background: black; color: white;"
							 "border: 2px solid black; selection-background-color: #bad8ff; selection-color: black;");
		
}

void
Puzzle::toUpper()
{ 
	for (int i = 0; i < 7; i++) {
		le[i]->setText(le[i]->text().toUpper()); 
		le[i]->setModified(true);
	}
}

QString
Puzzle::letrs() const
{ 
	QString puzzle;

	for (int i = 0; i < 7; i++) {
		if (!le[i]->isModified())
			return "";

		puzzle[i] = (le[i]->text())[0].toLower();
		if (!puzzle[i].isLower()) 
			return "";
	}

	return puzzle;
}

void 
Puzzle::paintEvent(QPaintEvent *ev)
{
	QPainter p(this);
	p.setRenderHints(QPainter::Antialiasing);
	p.setPen(Qt::black);
	p.setBrush(QBrush(QColor("snow")));
	p.drawPolygon(points, 6);		
	p.setBrush(QBrush(Qt::black));
	p.drawPolygon(points2, 6);		

	for (int i = 0; i < 6; i++)
		p.drawLine(QLineF(points[i], points2[i]));

	ev->accept();
}

void 
Puzzle::mousePressEvent(QMouseEvent *ev)
{
	const QPoint pt = ev->pos();
	if (p1->containsPoint(pt, Qt::OddEvenFill)) {
		le[0]->setFocus();
		le[0]->selectAll();
	} else if (p2->containsPoint(pt, Qt::OddEvenFill)) {
		le[1]->setFocus();
		le[1]->selectAll();
	} else if (p3->containsPoint(pt, Qt::OddEvenFill)) {
		le[2]->setFocus();
		le[2]->selectAll();
	} else if (p4->containsPoint(pt, Qt::OddEvenFill)) {
		le[3]->setFocus();
		le[3]->selectAll();
	} else if (p5->containsPoint(pt, Qt::OddEvenFill)) {
		le[4]->setFocus();
		le[4]->selectAll();
	} else if (p6->containsPoint(pt, Qt::OddEvenFill)) {
		le[5]->setFocus();
		le[5]->selectAll();
	} else if (p7->containsPoint(pt, Qt::OddEvenFill)) {
		le[6]->setFocus();
		le[6]->selectAll();
	}

	ev->accept();
}
